------------------------------------------------------
--                                                  --
--  Ernst-Abbe-Fachhochschule Jena                  --
--  Fachbereich Elektrotechnik/Informationstechnik  --
--  Programmierbare Logik                           --
--                                                  --
--  Projektarbeit Equalizer                         --
--  Juergen Doeffinger                              --
--  Christoph Dieck                                 --
--                                                  --
--  Equalizer.vhd                                   --
--  Version 2012-06-09-01                           --
--                                                  --
------------------------------------------------------



-- Bibliothken einbinden
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;



entity Equalizer is
	port 
	(
		-- inputs
		CLOCK_50 	: in std_logic; 							-- 50 MHz Takt vom Board 
		CLOCK_27 	: in std_logic; 							-- 27 MHz Takt vom Board
		KEY		 	: in std_logic_vector(3 downto 0);	-- Taster zum auswählen der Filter und Einstellen der Verstärkungsfaktoren
		SW		 	: in std_logic_vector(17 downto 0);		-- Schalter zum Einstellen verschiedener Funktionen, wie z.B: Mute
		AUD_ADCLRCK : in std_logic;							-- Kennzeichnung ob Daten vom linken oder rechten Kanal vom ADC übertragen werden
		AUD_ADCDAT 	: in std_logic;							-- serielle Datenübertragungsschnittstelle vom ADC
		AUD_BCLK 	: in std_logic;							-- 12 MHz Takt des Audio Codecs
		AUD_DACLRCK : in std_logic;							-- Kennzeichnung ob Daten vom linken oder rechten Kanal vom DAC übertragen werden
		
		-- outs
		LCD_EN 		: out std_logic; 									-- LCD Ein-/Auschaltsignal
		LCD_RW 		: out std_logic; 									-- Schreib-/Lesesignal an LCD
		LCD_RS 		: out std_logic; 									-- RS LCD
		LCD_ON 		: out std_logic; 									-- Ein-/Ausschaltsignal LCD komplett
		LCD_BLON 	: out std_logic; 									-- Ein-/Ausschaltsignal Hintergrundbeleuchtung LCD
		HEX0			: out unsigned(6 downto 0);					-- 7-Segment-Anzeige Buchstabe 'b'
		HEX1			: out unsigned(6 downto 0);					-- 7-Segment-Anzeige Buchstabe 'd'
		HEX2			: out unsigned(6 downto 0);					-- 7-Segment-Anzeige Verstärkungsfaktor Einer-Stelle
		HEX3			: out unsigned(6 downto 0);					-- 7-Segment-Anzeige Verstärkungsfaktor Zehner-Stelle
		HEX4			: out unsigned(6 downto 0);					-- 7-Segment-Anzeige Verstärkungsfaktor Vorzeichen
		HEX5			: out unsigned(6 downto 0) := "1111111";	-- bleibt aus; wird nicht verwendet
		HEX6			: out unsigned(6 downto 0);					-- 7-Segment-Anzeige Filterauswahl Einer-Stelle
		HEX7			: out unsigned(6 downto 0);					-- 7-Segment-Anzeige Filterauswahl Zehner-Stelle
		I2C_SCLK 	: out std_logic;									-- Takt des I2C Audio Codec Interface
		AUD_DACDAT 	: out std_logic;									-- serielle Datenübertragungsschnittstelle zum DAC
		AUD_XCK 		: out std_logic;									-- Takteingang des Audio Codec (hier mit 12 MHz getaktet)

		-- inouts
		LCD_DATA : inout unsigned(7 downto 0);		-- LCD - Daten BUS
		I2C_SDAT : inout std_logic						-- serieller Daten BUS des I2C Audio Codec Interface
	);	
end Equalizer;



architecture behavior of Equalizer is

	-- Taktgenerator für die verschiedenen Takte einbinden
	component clock_generator
		port
		(
			-- inputs
			CLOCK_50 : in std_logic; -- Eingang 50 MHz Takt
			CLOCK_27 : in std_logic; -- Eingang 27 MHz Takt
			rst		: in std_logic; -- Rücksetzeingang
		
			-- outputs
			clk_50MHz 	: out std_logic; -- Ausgang 50 MHz Takt PLL gesteuert
			clk_27MHz 	: out std_logic; -- Ausgang 27 MHz Takt PLL gesteuert
			clk_12MHz 	: out std_logic; -- Ausgang 12 MHz Takt PLL gesteuert
			clk_400kHz 	: out std_logic; -- Ausgang 400 kHz Takt PLL gesteuert nach clock_divider
			clk_100Hz 	: out std_logic; -- Ausgang 100 Hz Takt PLL gesteuert nach clock_divider
			reset			: out std_logic 
		); 
	end component;
	
	-- Taktsignale
	signal clk_50MHz  : std_logic; -- 50 MHz Takt
	signal clk_27MHz  : std_logic; -- 27 MHz Takt
	signal clk_12MHz  : std_logic; -- 12 MHz Takt
	signal clk_400kHz : std_logic; -- 400 kHz Takt
	signal clk_100Hz  : std_logic; -- 100 Hz Takt
	signal reset 		: std_logic := '1';
	
	
	-- MMI einbinden
	component MMI
		port
		(
			-- inputs
			clk_50MHz 		: in std_logic;		-- 50 MHz Takt
			reset				: in std_logic; 		-- Rücksetzeingang
			next_filter 	: in std_logic;		-- nächsten Filter auswählen
			prior_filter 	: in std_logic;		-- vorherigen Filter auswählen
			inc 				: in std_logic;		-- Verstärkungsfaktor Filter/Volume inkrementieren (1dB)
			dec 				: in std_logic;		-- Verstärkungsfaktor Filter/Volume dekrementieren (-1dB)
			seg_off			: in std_logic;		-- 7-Segement-Anzeigen ein-/ausschalten (0 -> ein, 1 -> aus);
			jd					: in std_logic;
		
			-- outputs
			volume 		: out signed(7 downto 0);		-- Verstärkungsfaktor Lautstärke in dB
			filter1 		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter1 in dB
			filter2 		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter2 in dB
			filter3		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter3 in dB
			filter4		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter4 in dB
			filter5		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter5 in dB
			filter6		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter6 in dB
			filter7		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter7 in dB
			filter8		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter8 in dB
			filter9		: out signed(7 downto 0);		-- Verstärkungsfaktor Filter9 in dB
			filter10 	: out signed(7 downto 0);		-- Verstärkungsfaktor Filter10 in dB
			filter11 	: out signed(7 downto 0);		-- Verstärkungsfaktor Filter11 in dB
			filter12 	: out signed(7 downto 0);		-- Verstärkungsfaktor Filter12 in dB
			filter13 	: out signed(7 downto 0);		-- Verstärkungsfaktor Filter13 in dB
			filter14 	: out signed(7 downto 0);		-- Verstärkungsfaktor Filter14 in dB
			LCD_EN 		: out std_logic;					-- LCD Ein-/Auschaltsignal
			LCD_RW 		: out std_logic;					-- Schreib-/Lesesignal an LCD
			LCD_RS 		: out std_logic;					-- RS LCD
			LCD_ON 		: out std_logic;					-- Ein-/Ausschaltsignal LCD komplett
			LCD_BLON 	: out std_logic;					-- Ein-/Ausschaltsignal Hintergrundbeleuchtung LCD
			filter_ones : out unsigned(6 downto 0);	-- 7-Segment-Anzeige Auswahlfilter Einer-Stelle
			filter_ten 	: out unsigned(6 downto 0);	-- 7-Segment-Anzeige Auswahlfilter Zehner-Stelle
			db_ones 		: out unsigned(6 downto 0);	-- 7-Segment-Anzeige Verstärkungsfaktor Filter/Volume Einer-Stelle
			db_ten 		: out unsigned(6 downto 0);	-- 7-Segment-Anzeige Verstärkungsfaktor Filter/Volume Zehner-Stelle
			db_sign 		: out unsigned(6 downto 0);	-- 7-Segment-Anzeige Verstärkungsfaktor Filter/Volume Vorzeichen
			db_letter_1 : out unsigned(6 downto 0);	-- 7-Segement-Anzeige Buchstebe 'd' für db
			db_letter_2 : out unsigned(6 downto 0);	-- 7-Segement-Anzeige Buchstebe 'b' für db
	
			-- inouts
			LCD_DATA : inout unsigned(7 downto 0)	-- LCD - Daten BUS
		);
	end component;
	
	-- Verstärkungsfaktoren
	signal volume 	: signed(7 downto 0);		-- Verstärkungsfaktor Lautstärke in dB
	signal filter1	: signed(7 downto 0);		-- Verstärkungsfaktor Filter1 in dB
	signal filter2 : signed(7 downto 0);		-- Verstärkungsfaktor Filter2 in dB
	signal filter3	: signed(7 downto 0);		-- Verstärkungsfaktor Filter3 in dB
	signal filter4	: signed(7 downto 0);		-- Verstärkungsfaktor Filter4 in dB
	signal filter5	: signed(7 downto 0);		-- Verstärkungsfaktor Filter5 in dB
	signal filter6	: signed(7 downto 0);		-- Verstärkungsfaktor Filter6 in dB
	signal filter7	: signed(7 downto 0);		-- Verstärkungsfaktor Filter7 in dB
	signal filter8	: signed(7 downto 0);		-- Verstärkungsfaktor Filter8 in dB
	signal filter9	: signed(7 downto 0);		-- Verstärkungsfaktor Filter9 in dB
	signal filter10 : signed(7 downto 0);		-- Verstärkungsfaktor Filter10 in dB
	signal filter11 : signed(7 downto 0);		-- Verstärkungsfaktor Filter11 in dB
	signal filter12 : signed(7 downto 0);		-- Verstärkungsfaktor Filter12 in dB
	signal filter13 : signed(7 downto 0);		-- Verstärkungsfaktor Filter13 in dB
	signal filter14 : signed(7 downto 0);		-- Verstärkungsfaktor Filter14 in dB
	
	
	-- Audio Unit einbinden
	component audio_unit
		port
		(
			-- inputs
			clk_12MHz 	: in std_logic;				-- 12 MHz Takt
			clk_400kHz 	: in std_logic;				-- 400 kHz Takt
			clk_50MHz 	: in std_logic;				-- 50 MHz Takt
			AUD_ADCLRCK : in std_logic;				-- Kennzeichnung ob Daten vom linken oder rechten Kanal vom ADC übertragen werden
			AUD_ADCDAT 	: in std_logic;				-- serielle Datenübertragungsschnittstelle vom ADC
			AUD_BCLK 	: in std_logic;				-- 12 MHz Takt des Audio Codecs
			AUD_DACLRCK : in std_logic;				-- Kennzeichnung ob Daten vom linken oder rechten Kanal vom DAC übertragen werden
			MUTE 			: in std_logic;				-- vollständige Abschaltung des Tons (0 -> Ton aus; 1 -> Ton ein)
			volume 		: in signed(7 downto 0);	-- Verstärkungsfaktor Lautstärke in dB
			filter1 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter1 in dB
			filter2 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter2 in dB
			filter3 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter3 in dB
			filter4 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter4 in dB
			filter5 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter5 in dB
			filter6 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter6 in dB
			filter7 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter7 in dB
			filter8 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter8 in dB
			filter9 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter9 in dB
			filter10 	: in signed(7 downto 0);	-- Verstärkungsfaktor Filter10 in dB
			filter11 	: in signed(7 downto 0);	-- Verstärkungsfaktor Filter11 in dB
			filter12 	: in signed(7 downto 0);	-- Verstärkungsfaktor Filter12 in dB
			filter13 	: in signed(7 downto 0);	-- Verstärkungsfaktor Filter13 in dB
			filter14 	: in signed(7 downto 0);	-- Verstärkungsfaktor Filter14 in dB
			bypass 			: in std_logic;				-- Eingang nur zu Testzwecken (derzeit bei 0 -> normal und 1 -> IIR-Filter)
			reset			: in std_logic;				-- Rücksetzeingang
		
			-- outputs
			I2C_SCLK 	: out std_logic;			-- Takt des I2C Audio Codec Interface
			AUD_DACDAT 	: out std_logic;			-- serielle Datenübertragungsschnittstelle zum DAC
			AUD_XCK 		: out std_logic;			-- Takteingang des Audio Codec (hier mit 12 MHz getaktet)
		
			-- inouts
			I2C_SDAT : inout std_logic				-- serieller Daten BUS des I2C Audio Codec Interface
		);
	end component;
	

begin
	
	-- Taktgenerator Ports zuweisen
	clock_generator_1 : clock_generator port map 
	(
		CLOCK_50, 
		CLOCK_27, 
		SW(17),
		clk_50MHz, 
		clk_27MHz, 
		clk_12MHz, 
		clk_400kHz, 
		clk_100Hz,
		reset
	);
	
	
	-- MMI Ports zuweisen
	MMI_1 : MMI port 	map
	(
		clk_50MHz,
		reset,
		KEY(2),
		KEY(3),
		KEY(0),
		KEY(1),
		SW(1),
		SW(3),
		volume,
		filter1,
		filter2,
		filter3,
		filter4,
		filter5,
		filter6,
		filter7,
		filter8,
		filter9,
		filter10,
		filter11,
		filter12,
		filter13,
		filter14,
		LCD_EN,
		LCD_RW,
		LCD_RS,
		LCD_ON,
		LCD_BLON,
		HEX6,
		HEX7,
		HEX2,
		HEX3,
		HEX4,
		HEX1,
		HEX0,
		LCD_DATA
	);
	
	
	-- Audio Unit Ports zuweisen
	audio_unit_1 : audio_unit port map
	(
		clk_12MHz,
		clk_400kHz,
		clk_50MHz,
		AUD_ADCLRCK,
		AUD_ADCDAT,
		AUD_BCLK,
		AUD_DACLRCK,
		SW(0),
		volume,
		filter1,
		filter2,
		filter3,
		filter4,
		filter5,
		filter6,
		filter7,
		filter8,
		filter9,
		filter10,
		filter11,
		filter12,
		filter13,
		filter14,
		SW(2),
		reset,
		I2C_SCLK,
		AUD_DACDAT,
		AUD_XCK,
		I2C_SDAT
	);

end behavior;
