------------------------------------------------------
--                                                  --
--  Ernst-Abbe-Fachhochschule Jena                  --
--  Fachbereich Elektrotechnik/Informationstechnik  --
--  Programmierbare Logik                           --
--                                                  --
--  Projektarbeit Equalizer                         --
--  Juergen Doeffinger                              --
--                                                  --
--  audio_unit.vhd                                  --
--  Version 2012-06-15-01                           --
--                                                  --
------------------------------------------------------



-- Bibliothken einbinden
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;



entity audio_unit is
	port
	(
		-- inputs
		clk_12MHz 	: in std_logic;				-- 12 MHz Takt
		clk_400kHz 	: in std_logic;				-- 400 kHz Takt
		clk_50MHz 	: in std_logic;				-- 50 MHz Takt
		AUD_ADCLRCK : in std_logic;				-- Kennzeichnung ob Daten vom linken oder rechten Kanal vom ADC übertragen werden
		AUD_ADCDAT 	: in std_logic;				-- serielle Datenübertragungsschnittstelle vom ADC
		AUD_BCLK 	: in std_logic;				-- 12 MHz Takt des Audio Codecs
		AUD_DACLRCK : in std_logic;				-- Kennzeichnung ob Daten vom linken oder rechten Kanal vom DAC übertragen werden
		MUTE 			: in std_logic;				-- vollständige Abschaltung des Tons (0 -> Ton aus; 1 -> Ton ein)
		volume 		: in signed(7 downto 0);	-- Verstärkungsfaktor Lautstärke in dB
		filter1 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter1 in dB
		filter2 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter2 in dB
		filter3 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter3 in dB
		filter4 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter4 in dB
		filter5 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter5 in dB
		filter6 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter6 in dB
		filter7 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter7 in dB
		filter8 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter8 in dB
		filter9 		: in signed(7 downto 0);	-- Verstärkungsfaktor Filter9 in dB
		filter10 	: in signed(7 downto 0);	-- Verstärkungsfaktor Filter10 in dB
		filter11 	: in signed(7 downto 0);	-- Verstärkungsfaktor Filter11 in dB
		filter12 	: in signed(7 downto 0);	-- Verstärkungsfaktor Filter12 in dB
		filter13 	: in signed(7 downto 0);	-- Verstärkungsfaktor Filter13 in dB
		filter14 	: in signed(7 downto 0);	-- Verstärkungsfaktor Filter14 in dB
		bypass		: in std_logic;				-- Bypass Signaleingang (0 -> EQ ein; 1 -> EQ aus)
		reset			: in std_logic;				-- Rücksetzeingang
	
		-- outputs
		I2C_SCLK 	: out std_logic;	-- Takt des I2C Audio Codec Interface
		AUD_DACDAT 	: out std_logic;	-- serielle Datenübertragungsschnittstelle zum DAC
		AUD_XCK 		: out std_logic;	-- Takteingang des Audio Codec (hier mit 12 MHz getaktet)
	
		-- inouts
		I2C_SDAT : inout std_logic		-- serieller Daten BUS des I2C Audio Codec Interface
	);
end audio_unit;



architecture behavior of audio_unit is

	-- i2c_master dient zum Initialisieren des Audio-Codecs
	component i2c_master
		port
		(
			reset		: in std_logic;		--Asynchroner Reset
			CLOCK_50	: in std_logic; 		--Systemtakt (12.5MHz)
			CLK_EN	: in std_logic; 		--Takt zur Taktgenerierung am SCLK und Aktualisierung der Daten an SDIN
			SDIN 		: inout std_logic; 	--Datenausgang (Eingang für das Acknowledge-Bit)
			SCLK 		: out std_logic; 		-- Takt für den I2C-Bus (Hälfte der Frequenz an CLK_EN
        
			--Zu Diagnosezwecken
			DONE			: out std_logic := '0'; 				--Ausgang DONE wird bei vollständiger erfolgreicher Parametrierung gesetzt
			state_count	: out std_logic_vector(9 downto 0) 	--Zählerstand der durchlaufenen Zustande
		);
	end component;
	
	-- Signale zur Übernahme bzw. Übergabe von Signalen der Komponente i2c_master
	signal D			: std_logic := '0'; 					
	signal sc 		: std_logic_vector(9 downto 0); 	
	
	-- Komponente audio_input_register dient zum empfangen der seriell vom ADC
	-- gesendeten Daten und der Aufbereitung zur weiteren Verarbeitung in paralleler Struktur.
	component audio_input_register
		port 
		(
			-- inputs
			adclrc	: in std_logic;	-- 0 -> right channel; 1 -> left channel
			adcdat	: in std_logic;	-- serial audio data
			bclk		: in std_logic;	-- clock
		
			-- outputs
			dl		: out signed(23 downto 0);		-- parallel audio output left channel
			dr		: out signed(23 downto 0); 	-- parallel audio output right channel
			enl	: out std_logic;					-- read audio data left channel enable
			enr	: out std_logic					-- read audio data right channel enable
		);
	end component;
	
	-- Die Komponente filter_unit dient der Berechnung des Ausgangssignales 
	-- je nach Einstellungen der Filterbänke.
	component filter_unit 
		port 
		(
			-- inputs
			clk_50MHz	: in std_logic;					-- 50 MHz Takteingang
			dl				: in signed(23 downto 0);		-- Dateneingang linker Kanal
			dr				: in signed(23 downto 0);		-- Dateneingang rechter Kanal
			enl 			: in std_logic;					-- Freigabesignal zur Übernahme der Daten am Dateneingang linker Kanal
			enr 		 	: in std_logic;					-- Freigabesignal zur Übernahme der Daten am Dateneingang rechter Kanal
			vol		  	: in signed(7 downto 0);		-- Verstärkungsfaktor Lauftstärke in dB				
			g1 		 	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
			g2			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
			g3			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
			g4			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
			g5			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
			g6			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
			g7			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
			g8			  	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
			g9 		 	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
			g10		 	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
			g11		 	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
			g12 			: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
			g13		 	: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB		
			g14 			: in signed(7 downto 0);		-- Verstärkungsfaktor Filter 1 in dB
			bypass		: in std_logic;					-- Bypass Signaleingang (0 -> EQ ein; 1 -> EQ aus)
		
			-- outputs
			qdl		: out signed(23 downto 0);			-- Datenausgang linker Kanal		
			qdr		: out signed(23 downto 0);			-- Datenausgang rechter Kanal
			enql		: out std_logic;						-- Freigabesignal zur Übernahme der Daten am Datenausgang linker Kanal	
			enqr		: out std_logic						-- Freigabesignal zur Übernahme der Daten am Datenausgang rechter Kanal	
		);
	end component;

	-- Signale für die Komponente filter_unit zur Übergabe bzw. Übernahme entsprechender Signale
	signal dl	: signed(23 downto 0);		
	signal dr	: signed(23 downto 0);		
	signal enl	: std_logic;							
	signal enr	: std_logic;		
	signal ql	: signed(23 downto 0);		
	signal qr	: signed(23 downto 0);		
	signal enql	: std_logic;							
	signal enqr	: std_logic;				
	
	-- Komponente audio_output_register wandelt die parallelen Audiosignale 
	-- in serielle Daten um und sendet diese an den DAC.
	component audio_output_register
    port 
	(
			-- inputs
			dl			: in signed(23 downto 0);	-- parallel audio data left channel
			dr			: in signed(23 downto 0);	-- parallel audio data right channel
			enl		: in std_logic;				-- read left channel enable
			enr		: in std_logic;				-- read right channel enable
			bclk		: in std_logic;				-- clock
			daclrc	: in std_logic;				-- 0 -> right channel; 1 -> left channel
		
			-- outputs
			dacdat 	: out std_logic				-- serial audio data
		);
	end component;
	
	-- Signal zur Übernahme der Audiodaten vom audio_output_register zur 
	-- Übergabe an den DAC 
	signal dacdat : std_logic;	
	
begin

	-- i2c_master Ports zuweisen
	i2c_master_1 : i2c_master port map
	(
		reset,
		clk_12MHz,
		clk_400kHz,
		I2C_SDAT,
		I2C_SCLK,
		D,
		sc
	);

	-- audio_input_register Ports zuweisen
	audio_input_register_1 : audio_input_register port map
	(
		AUD_ADCLRCK,
		AUD_ADCDAT,
		AUD_BCLK,
		dl,
		dr,
		enl,
		enr
	);
	
	-- filter_unit Ports zuweisen
	filter_unit_1 : filter_unit port map
	(
		clk_50MHz,
		dl,
		dr,
		enl,
		enr,
		volume,
		filter1,
		filter2,
		filter3,
		filter4,
		filter5,
		filter6,
		filter7,
		filter8,
		filter9,
		filter10,
		filter11,
		filter12,
		filter13,
		filter14,
		bypass,
		ql,
		qr,
		enql,
		enqr
	);
	
	-- audio_output_register Ports zuweisen
	audio_output_register_1 : audio_output_register port map
	(
		ql,
		qr,
		enql,
		enqr,
		AUD_BCLK,
		AUD_DACLRCK,
		dacdat
	);
	
	AUD_XCK <= clk_12MHz;					-- 12 MHz Takt an den Audio Codec weitergeben für Taktgenerierung (z.B. 48 kHz Abtastfrequenz)
	AUD_DACDAT <= dacdat and not MUTE;	-- UND-Verknüpfung der Audio-Ausgangsdaten mit dem MUTE-Signal zur schenllen Abschaltung der Tonwiedergabe
END behavior;
