------------------------------------------------------
--                                                  --
--  Ernst-Abbe-Fachhochschule Jena                  --
--  Fachbereich Elektrotechnik/Informationstechnik  --
--  Programmierbare Logik                           --
--                                                  --
--  Projektarbeit Equalizer                         --
--  Juergen Doeffinger                              --
--  Christoph Dieck                                 --
--                                                  --
--  clock_generator.vhd                             --
--  Version 2012-06-16-01                           --
--                                                  --
------------------------------------------------------



-- Bibliothken einbinden
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;



entity clock_generator is
	port
	(
		-- inputs
		CLOCK_50 : in std_logic; 			-- Eingang 50 MHz Takt
		CLOCK_27 : in std_logic; 			-- Eingang 27 MHz Takt
		rst		: in std_logic := '1'; 	-- Rücksetzeingang
		
		-- outputs
		clk_50MHz 	: out std_logic; 			-- Ausgang 50 MHz Takt PLL gesteuert
		clk_27MHz 	: out std_logic; 			-- Ausgang 27 MHz Takt PLL gesteuert
		clk_12MHz 	: out std_logic; 			-- Ausgang 12 MHz Takt PLL gesteuert
		clk_400kHz 	: out std_logic; 			-- Ausgang 400 kHz Takt PLL gesteuert nach clock_divider
		clk_100Hz 	: out std_logic;  		-- Ausgang 100 Hz Takt PLL gesteuert nach clock_divider
		reset			: out std_logic := '1' 	-- Ausgang zur Übergabe eines Resets an die anderen Komponenten
	); 
end clock_generator;


architecture behavior of clock_generator is

	-- PLL 1 zur Takterzeugung von 10 MHz und Aufbereitung der 50 MHz einbinden
	component EQ_PLL_50
	PORT
	(
		inclk0	: in  std_logic  := '0';	-- 50 MHz Takteingang
		c0			: out std_logic ;				-- 50 MHz Taktausgang
		c1			: out std_logic 				-- 10 MHz Taktausgang
	);
	end component;
	
	-- 10 MHz Taktsignal zur Weiterführung an den clock_divider
	signal clk_10MHz : std_logic;		
	
	-- PLL 2 zur Takterzeugung von 12 MHz und Aufbereitung der 27 MHz einbinden
	component EQ_PLL_27
	PORT
	(
		inclk0	: in  std_logic  := '0';	-- 27 MHz Takteingang
		c0			: out std_logic ;				-- 27 MHz Taktausgang
		c1			: out std_logic 				-- 12 MHz Taktausgang
	);
	end component;
	
	-- Taktteiler einbinden zur Erzeugung des 400 kHz und 100 Hz Taktes
	component clock_divider
		port
		(
			-- inputs
			clk_10MHz 	: in std_logic;					-- 10 MHz Takteingang
		
			-- outputs
			clk_400kHz : out std_logic := '0';		-- 400 kHz Taktausgang
			clk_100Hz  : out std_logic := '0'		-- 100 Hz Taktausgang
		);
	end component;

	-- Der start counter ist nach dem Einschalten des Gerätes aktiv.
	-- Für ca. 2 Sekunden wird ein Reset-Signal ausgegeben, um
	-- einen stabilen Zustand zu haben, bevor die Schaltung
	-- aktiv wird.
	signal start_counter : unsigned(25 downto 0) := (others => '1');

begin

	-- PLL 1 Ports zuweisen
	pll_1 : EQ_PLL_50 port map (CLOCK_50, clk_50MHz, clk_10MHz);

	-- PLL 2 Ports zuweisen
	pll_2 : EQ_PLL_27 port map (CLOCK_27, clk_27MHz, clk_12MHz);
	
	-- Taktteiler Ports zuweisen
	clock_divider_1 : clock_divider port map (clk_10MHz, clk_400kHz, clk_100Hz);

	-- Dieser Prozess sorgt dafür, dass bei einem Eingang eines
	-- Reset-Signals bzw. nach dem Einschalten des Gerätes
	-- für ca. 2 Sekunden eine Reset-Signal an die anderen
	-- Kompoonenten gesendet wird.
	reset_workflow: process begin
		wait until rising_edge(CLOCK_27);
		if (rst='1') then start_counter <= (others => '1'); end if;
		if (start_counter > 0) then  
			start_counter <= start_counter - 1; 
			reset <= '1';
		else 
			reset <= '0';
		end if;
	end process;

end behavior;
