--	Funktionsbeschreibung:
--	Die Entität parametriert den Audio-Codec über das I2C-Protokoll in der Weise, 
--	dass die für die Anwendung nötigen Einstellungen gemacht werden.
--	Hierbei erzeugt eine State Machine den notwendigen clock (SCLK) und legt zeitlich abgestimmt die Daten auf SDIN.
--	Das Acknowledge-Bit, das der Audio-Codec nach jedem vollständigen Datenbyte wird ausgewertet und bei Fehlern in 
--	den Zustand "error" gewechselt. Der Zustandszähler gibt dann den Schritt aus, bei dem der Fehler aufgetreten ist.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_signed.all;
use ieee.numeric_std.all;

ENTITY i2c_master IS
	PORT (
				reset 	: IN STD_LOGIC := '1';	--Asynchroner Reset
				CLOCK_50	: IN STD_LOGIC; 			--Systemtakt (12.5MHz)
				CLK_EN	: IN STD_LOGIC; 			--Takt zur Taktgenerierung am SCLK und Aktualisierung der Daten an SDIN
				SDIN 		: INOUT STD_LOGIC; 		--Datenausgang (Eingang für das Acknowledge-Bit)
				SCLK 		: OUT STD_LOGIC; 			-- Takt für den I2C-Bus (Hälfte der Frequenz an CLK_EN
        
				--Zu Diagnosezwecken
				DONE: OUT STD_LOGIC := '0'; 							--Ausgang DONE wird bei vollständiger erfolgreicher Parametrierung gesetzt
				state_count: OUT STD_LOGIC_VECTOR (9 downto 0) 	--Zählerstand der durchlaufenen Zustände
			);
	END i2c_master;

ARCHITECTURE BEHAVIOR OF i2c_master IS
	
	TYPE type_fstate IS (anfangszustand,startbedingung,bit_n,taktende,ack_warten,ack_warten2,ack,error,stop_warten,stopbedingung,ende);
	SIGNAL reg_fstate : type_fstate;
	SIGNAL state_count_int: unsigned (9 downto 0) := to_unsigned(0, 10); --Signal zum Speichern der Status-Anzahl
	type array_typ is array (natural range <>) of std_logic_vector(23 downto 0); --Array für Befehle erzeugen

BEGIN
	
	--Statusanzahl mit Ausgang verbinden
	state_count <= std_logic_vector(state_count_int);
		
	PROCESS (reset, CLOCK_50)

	--Der Automat legt die Informationen aus der Konstanten befehlsregister bitweise auf SDIN
	--Neben der Start- und Stopbedingung wird nach jedem übertragenem Byte das Acknowledge-Bit ausgewertet
	
	--Der Clock für den I2C-Bus wird durch SCLK repräsentiert, der etwa bei jedem zweiten Takt von CLK_EN einen Takt ausgibt
	--Hierdurch werden die etwa 25kHz für den I2C-Bus bereitgestellt
    
	constant befehlsregister : array_typ := (
	--Die Pins CSB und MODE des Audio-Codecs sind fest verdrahtet auf GND geführt, somit muss das 2-wire-interface (i2c) verwendet werden.
	--Die Adresse des Audio-Codecs ist auf 0011010 festgelegt.
    
	--Der grundsätzliche Aufbau der Telegramme ergibt sich daher:
	--7 bit Bus-Adresse, 1 bit "write", 7 bit Register-Adresse, 9 bit Daten
	-- B"00110100_000XXXX_XXXXXXXXX"  

	B"00110100_0000000_000000000", 	--	Dummy
	B"00110100_0001111_000000000", 	--	Reset
	B"00110100_0000000_000010111",	--	Left Line In
	B"00110100_0000001_000010111", 	-- Right Line In
	B"00110100_0000010_001111001",	--	Left Headphone Out
	B"00110100_0000011_001111001",	--	Right Headphone Out
	B"00110100_0000100_000010010",	--	Analog Audio Path Control
	B"00110100_0000101_000010000",	--	Digital Audio Path Control
	B"00110100_0000110_000000010",	-- Power Down Control
	B"00110100_0000111_001001001", 	-- Digital Audio Interface Format
	B"00110100_0001000_000000001", 	--	Sampling Control (h1001 -> 48 kHz / 48 kHz im USB-Mode bei 12 MHz Takt;	)
	B"00110100_0001001_000000001");	--	activate Audio Interface
	
	variable Byte : integer range 0 to 8; 		--Blocklänge zwischen zwei Acknowledge-Bits
	variable Sektion : integer range 0 to 3; 	--Anzahl der Bytes eines Befehls
	variable Befehls_NR : integer range 0 to befehlsregister'Length := 0; --Anzahl der Befehle, die versendet werden müssen

	BEGIN

		IF (reset='1') THEN
			--Zurücksetzen aller Variablen, damit der Parametrierung des Audio-Codecs neu starten kann
			reg_fstate <= anfangszustand;
			SDIN <= '0';
			SCLK <= '0';
			DONE <= '0';
			Befehls_NR := 0;
			state_count_int <= to_unsigned(2, 10);
		ELSIF (rising_edge(CLOCK_50)) THEN
			IF (CLK_EN = '1') THEN
				CASE reg_fstate IS
					WHEN anfangszustand => 
						--notwendiger Anfangszustand, damit die Startbedingung für die Übertragung erfolgen kann
						reg_fstate <= startbedingung;
						SDIN <= '1';
						SCLK <= '1';
						Sektion := 2; --Anfangswert der Variable Sektion (2+1 Byte pro zu �bertragenden Parametersatz)
					WHEN startbedingung =>
						--negative Flanke an SDIN w�hrend SCLK high-Pegel hat (Startbedingung)
						reg_fstate <= bit_n;
						SDIN <= '0';
						SCLK <= '1';
						Byte := 7; --Anfangswert der Variable Byte (7+1 Bit pro zu �bertragenden Parametersatz
					WHEN bit_n =>
						--legen der einzelnen Bits auf SDIN, bei der negativen Flanke an SCLK
						reg_fstate <= taktende;
						SDIN <= befehlsregister(Befehls_NR)(8 * Sektion + Byte); --Bitweise Zugriff auf die Konstante Befehlsregister
						SCLK <= '0';
					WHEN taktende =>
						--das einzelne Bit liegt weiterhin an SDIN an und die positive Flanke an SCLK kommt (Bedingung f�r I2C)
						IF (Byte > 0) THEN
							reg_fstate <= bit_n;
						ELSE
							reg_fstate <= ack_warten;  
						END IF;
						SDIN <= befehlsregister(Befehls_NR)(8 * Sektion + Byte);
						SCLK <= '1';
						Byte := Byte - 1; --Dekrement der Variable Byte, damit das n�chste Bit gesendet werden kann
					WHEN ack_warten =>
						--SDIN hochohmig schalten, damit bei der Codec bei der n�chsten positiven Flanke den Buspegel auf low ziehen kann
						reg_fstate <= ack_warten2;
						SDIN <= 'Z';
						SCLK <= '0';
					WHEN ack_warten2 =>
						--positive Flake an SCLK, beibehaltung des hochohmigen Zustandes an SDIN
						reg_fstate <= ack;
						SDIN <= 'Z';
						SCLK <= '1';
					WHEN ack =>
						--Abfrage des Buspegels an SDIN
						IF (SDIN = '0') THEN
							IF (Sektion > 0) THEN
								reg_fstate <= bit_n;
							ELSE
								reg_fstate <= stop_warten;
							END IF;
						ELSE
							reg_fstate <= error; --Zustand "error", wenn das Acknowledge-Bit nicht gesetzt wurde
						END IF;
						SDIN <= 'Z';
						SCLK <= '0';
						Byte := 7;
						Sektion := Sektion - 1; --Dekrement der Variablen Sektion, damit das n�chste Byte �bertragen werden kann
					WHEN error =>
						--Endg�ltiger Zustand, wenn ein Fehler aufgetreten ist
						reg_fstate <= error;
						SDIN <= '0';
						SCLK <= '0';
					WHEN stop_warten =>
						--Vorbereiten der Stopbedingung, durch setzen des Low-Pegels an beiden Ausg�ngen
						reg_fstate <= stopbedingung;
						SDIN <= '0';
						SCLK <= '0';
					WHEN stopbedingung =>
						--Positive Flanke an SCLK w�hrend SDIN low-Pegel hat (Stoppbedingung)
						IF (Befehls_NR < befehlsregister'Length) THEN
							--Wiederholung des gesamten Zyklus, bis alle Informationen an den Audio-Codec �bertragen wurden
							reg_fstate <= anfangszustand;
						ELSE
							reg_fstate <= ende;
						END IF;
						SDIN <= '0';
						SCLK <= '1';
						Befehls_NR := Befehls_NR + 1; --Dekrement der Variable Befehls_NR, damit der n�chste Befehl gesendet werden kann
					WHEN ende =>
						--Endg�ltiger Zustand, wenn alles korrekt �bertragen wurde
						reg_fstate <= ende;
						SDIN <= '1';
						SCLK <= '1';
						DONE <= '1';
					WHEN OTHERS => 
						SDIN <= 'X';
						SCLK <= 'X';
						report "Reach undefined state";
				END CASE;
				
				IF ((reg_fstate /= ende) AND (reg_fstate /= error)) THEN 
					--Status-Z�hler erh�hen, wenn kein endg�ltiger Zustand erreicht ist
					state_count_int <= state_count_int + 1;
				END IF;
			END IF;
        END IF;
    END PROCESS;
END BEHAVIOR;
