------------------------------------------------------
--                                                  --
--  Ernst-Abbe-Fachhochschule Jena                  --
--  Fachbereich Elektrotechnik/Informationstechnik  --
--  Programmierbare Logik                           --
--                                                  --
--  Projektarbeit Equalizer                         --
--  Juergen Doeffinger                              --
--  Christoph Dieck                                 --
--                                                  --
--  mmi_controller.vhd                              --
--  Version 2012-06-13-01                           --
--                                                  --
------------------------------------------------------



-- Bibliothken einbinden
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;



entity lcd_controller is
	port
	(
			-- inputs
			clk_50MHz	: in std_logic;						-- 50 MHz Takt
			reset			: in std_logic;						-- Rücksetzeingang
			lcd_1			: in unsigned(11 downto 0);		-- Zeichen 1 zur Anzeige des ausgewühltn Filters
			lcd_2			: in unsigned(11 downto 0);		-- Zeichen 2 zur Anzeige des ausgewühltn Filters
			lcd_3			: in unsigned(11 downto 0);		-- Zeichen 3 zur Anzeige des ausgewühltn Filters
			lcd_4			: in unsigned(11 downto 0);		-- Vorzeichen des Verstärkungsfaktors
			lcd_5			: in unsigned(11 downto 0);		-- Zehner-Stelle des Verstärkungsfaktors in dB
			lcd_6			: in unsigned(11 downto 0);		-- Einer-Stelle des Verstärkungsfaktors in dB

			-- outputs
			lcd_en		: out std_logic;	-- LCD Ein-/Auschaltsignal
			lcd_rw		: out std_logic;	-- Schreib-/Lesesignal an LCD
			lcd_rs		: out std_logic;	-- RS LCD
			lcd_on		: out std_logic;	-- Ein-/Ausschaltsignal LCD komplett
			lcd_blon	: out std_logic;		-- Ein-/Ausschaltsignal Hintergrundbeleuchtung LCD
			
			-- inouts
			lcd_data 	: inout unsigned(7 downto 0)	-- LCD - Daten BUS
	);
end lcd_controller;



architecture a_lcd_controller of lcd_controller is

	signal counter : unsigned(7 downto 0);		-- interner Zähler für die Cursorposition (welches Zeichen ist zu senden)
	signal output  : unsigned(11 downto 0);	-- beinhaltet das aktuell zu sendende Zeichen
	signal delay	: unsigned(19 downto 0);	-- interne Taktgenerierung 
	signal index	: unsigned(2 downto 0);		-- interner Zähler für den workflow
	signal start 	: boolean := true;			-- Übertragung ist gestartet
	signal stop 	: boolean := false;			-- Übertragung ist gestoppt

begin
		
	lcd_data  <= output(7 downto 0);		-- Daten an LCD übergeben
	lcd_rs 	 <= output(8);					-- RS auf entsprechenden Befehl einstellen
	lcd_rw    <= '0';							-- Schreibvorgang initiieren
	lcd_on    <= '1';							-- LCD einschalten bzw. eingeschaltet lassen
	lcd_blon  <= '1';							-- Hintergrundbeleuchtung einschalten bzw. eingeschaltet lassen.

	-- Der folgende Prozess verfolgt das Ueberschreiben des LCD mit neuen Werten bzw.
	-- bei erstmaliger Anwendung auch die Initialisierung des LCD - Displays.
	process begin
	
		-- Warten auf steigende Flanke des 50 MHz Taktes
		wait until rising_edge(clk_50MHz);
	
	
		if (reset='1') then 
			-- bei Reset alles zurücksetzen
			output 	<= x"000";
			counter 	<= x"00";
			delay 	<= x"00000";
			index 	<= "000";
			start 	<= true;
			stop 		<= false;
		else 
			-- Übertragung gestartet?
			if (not stop) then
				-- wenn ja -> Wartezeit abgelaufen?
				if ((start = true) and (delay < x"F4240")) then
					-- Wartezeit noch nicht abgelaufen, dann Wartezeitzähler inkrementieren
					delay <= delay + 1;
				else 
					--Wartezeit abgelaufen -> nächtes Zeichen senden
					start <= false;
					case index is
						when "000" => lcd_en <= '0'; index <= "001"; 	-- LCD freigabe löschen
						when "001" => case counter is							-- nächtes zu sendes Zeichen auswählen
										when x"00" => output <= x"038"; counter <= x"01";
										when x"01" => output <= x"00C"; counter <= x"02";
										when x"02" => output <= x"001"; counter <= x"03";
										when x"03" => output <= x"006"; counter <= x"04";
										when x"04" => output <= x"080"; counter <= x"05";
										when x"05" => output <= x"120"; counter <= x"06";
										when x"06" => output <= x"145"; counter <= x"07";
										when x"07" => output <= x"171"; counter <= x"08";
										when x"08" => output <= x"175"; counter <= x"09";
										when x"09" => output <= x"161"; counter <= x"0A";
										when x"0A" => output <= x"17C"; counter <= x"0B";
										when x"0B" => output <= x"169"; counter <= x"0C";
										when x"0C" => output <= x"17A"; counter <= x"0D";
										when x"0D" => output <= x"165"; counter <= x"0E";
										when x"0E" => output <= x"172"; counter <= x"0F";
										when x"0F" => output <= x"0C0"; counter <= x"10";
										when x"10" => output <= x"120"; counter <= x"11";
										when x"11" => output <= lcd_1;  counter <= x"12";
										when x"12" => output <= lcd_2;  counter <= x"13";
										when x"13" => output <= lcd_3;  counter <= x"14";
										when x"14" => output <= x"120"; counter <= x"15";
										when x"15" => output <= lcd_4;  counter <= x"16";
										when x"16" => output <= lcd_5;  counter <= x"17";
										when x"17" => output <= lcd_6;  counter <= x"18";
										when x"18" => output <= x"164"; counter <= x"19";
										when x"19" => output <= x"142"; counter <= x"0F";
										when others => counter <= x"00";
									end case;
									index <= "010";
						when "010" => lcd_en <= '1'; index <= "011"; delay <= x"00000";	-- lcd_en freigeben,
						when "011" => 	if (delay < 16) then 										-- nach entspechender Warte, damit das LCD die Daten übernehmen kann,
											delay <= delay + 1; 											-- lcd_en löschen
										else
											lcd_en <= '0';
											delay <= x"00000";
											index <= "100";
										end if;
						when "100" => 	if (delay < x"3FFFE") then		-- nach entsprechender Wartezeit -> Senden des nächsten Zeichens einleiten
											delay <= delay + 1;
										else 
											delay <= x"00000";
											index <= "000";
										end if;
						when others => null;
					end case;
				end if;
			end if;
		end if;
	end process;
end a_lcd_controller; 